@@ -1,6 +1,6 @@
 # gpxviewer
 #
-# Copyright (C) 2016-2018 Sergey Salnikov <salsergey@gmail.com>
+# Copyright (C) 2016-2019 Sergey Salnikov <salsergey@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3
@@ -20,13 +20,11 @@ from PyQt5.QtCore import QCoreApplication, QLocale, QTranslator
 from PyQt5.QtWidgets import QApplication
 import gpxviewer.rc_gpxviewer
 
-app = None
-
 
 def main():
-  global app
   app = QApplication(sys.argv)
-  QCoreApplication.setApplicationVersion('0.9.3')
+  QCoreApplication.setApplicationName('GPX Viewer')
+  QCoreApplication.setApplicationVersion('1.0')
 
   translator = QTranslator(app)
   translator.load(QLocale(), 'gpxviewer', '_', ':/translations')
