@@ -1,6 +1,6 @@
 # gpxviewer
 #
-# Copyright (C) 2016-2018 Sergey Salnikov <salsergey@gmail.com>
+# Copyright (C) 2016-2019 Sergey Salnikov <salsergey@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3
@@ -16,18 +16,18 @@
 
 import xml.etree.ElementTree as ET
 from datetime import datetime, timedelta
-from math import pi, sin, cos, acos
-from PyQt5 import QtCore
-from PyQt5.QtGui import QColor, QFont, QGuiApplication, QPixmap, QPainter
+from math import acos, cos, modf, pi, sin, sqrt
+from PyQt5.QtCore import Qt, QAbstractTableModel, QModelIndex, QObject, QPointF, QSortFilterProxyModel, pyqtSignal
+from PyQt5.QtGui import QColor, QFont, QGuiApplication, QPainter, QPainterPath, QPen, QPixmap, QPolygonF
 from gpxviewer.configstore import TheConfig
 
 
 WPTFIELDS = NAME, LAT, LON, ALT, DIST, TIME, TIMEDELTA, TIME_DAYS = range(8)
 TRKFIELDS = TRKNAME, TRKSEGS, TRKPTS, TRKLEN, TRKTIME, TRKDUR = range(6)
-ValueRole, IDRole, IncludeRole, SplitStateRole, MarkerRole, SplitLineRole, CaptionRole, NeglectRole = range(QtCore.Qt.UserRole, QtCore.Qt.UserRole + 8)
-INCSTATES = INC_DEFAULT, INC_SKIP, INC_MARKER, INC_CAPTION = range(4)
-INCCOLORS = DefaultColor, SkipColor, MarkerColor, CaptionColor = QtCore.Qt.white, QColor(255, 225, 225), QColor(225, 225, 255), QColor(225, 255, 225)
-MARKER_COLOR, MARKER_STYLE, MARKER_SIZE, LINE_COLOR, LINE_STYLE, LINE_WIDTH, CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE = range(9)
+ValueRole, IDRole, IncludeRole, MarkerRole, CaptionRole, SplitLineRole, NeglectRole, MarkerStyleRole, CaptionStyleRole, SplitLineStyleRole = range(Qt.UserRole, Qt.UserRole + 10)
+SkipColor, MarkerColor, CaptionColor = (QColor(255, 225, 225), QColor(225, 225, 255), QColor(225, 255, 225))
+MARKER_COLOR, MARKER_STYLE, MARKER_SIZE, CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE, LINE_COLOR, LINE_STYLE, LINE_WIDTH = \
+  ('MarkerColor', 'MarkerStyle', 'MarkerSize', 'CaptionPositionX', 'CaptionPositionY', 'CaptionSize', 'SplitLineColor', 'SplitLineStyle', 'SplitLineWidth')
 
 
 class GpxWarning(Exception):
@@ -35,32 +35,51 @@ class GpxWarning(Exception):
     super(GpxWarning, self).__init__(value)
 
 
-class WptModel(QtCore.QAbstractTableModel):
+class WptModel(QAbstractTableModel):
   def __init__(self, parent=None):
     super(WptModel, self).__init__(parent)
     self.fields = [self.tr('Name'), self.tr('Latitude'), self.tr('Longitude'), self.tr('Elevation'),
                    self.tr('Distance'), self.tr('Time'), self.tr('Time difference'), self.tr('Time in days')]
     self.resetModel()
     self.pix = QPixmap(16, 16)
-    self.pix.fill(QtCore.Qt.transparent)
+    self.pix.fill(Qt.transparent)
 
   def rowCount(self, parent=None):
-    return len(self.waypoints)
+    if parent is not None and parent.isValid():
+      return 0
+    else:
+      return len(self.waypoints)
 
   def columnCount(self, parent=None):
-    return len(self.fields)
+    if parent is not None and parent.isValid():
+      return 0
+    else:
+      return len(self.fields)
 
-  def data(self, index, role=QtCore.Qt.DisplayRole):
-    if role == QtCore.Qt.DisplayRole or role == QtCore.Qt.EditRole:
+  def data(self, index, role=Qt.DisplayRole):
+    if role == Qt.DisplayRole or role == Qt.EditRole:
       if index.column() == NAME and index.row() in self.changedNames:
-        return self.pointNames[self.changedNames.index(index.row())]
+        return self.changedNames[index.row()]
       elif index.column() == TIME and self.waypoints[index.row()][index.column()] != '':
-        return str(self.waypoints[index.row()][index.column()] + timedelta(minutes=TheConfig['ProfileStyle'].getint('TimeZoneOffset')))
+        return str(self.waypoints[index.row()][index.column()] + timedelta(minutes=TheConfig.getValue('ProfileStyle', 'TimeZoneOffset')))
+      elif index.column() == LAT or index.column() == LON:
+        if TheConfig.getValue('ProfileStyle', 'CoordinateFormat') == 0:  # Decimal degrees
+          return str(round(self.waypoints[index.row()][index.column()], 6))
+        elif TheConfig.getValue('ProfileStyle', 'CoordinateFormat') == 1:  # Degrees with decimal minutes
+          min, deg = modf(self.waypoints[index.row()][index.column()])
+          min = round(min * 60, 4)
+          return str(int(deg)) + '° ' + str(min) + '\''
+        else:  # Degrees, minutes, seconds
+          min, deg = modf(self.waypoints[index.row()][index.column()])
+          min = min * 60
+          sec, min = modf(min)
+          sec = round(sec * 60, 2)
+          return str(int(deg)) + '°' + str(int(min)) + '\'' + str(sec) + '"'
       else:
         return str(self.waypoints[index.row()][index.column()])
-    elif role == QtCore.Qt.DecorationRole and index.column() == NAME:
-      if index.data(IncludeRole) in {INC_MARKER, INC_CAPTION}:
-        return _markerIcon(index.data(MarkerRole)[MARKER_STYLE], index.data(MarkerRole)[MARKER_COLOR])
+    elif role == Qt.DecorationRole and index.column() == NAME:
+      if index.data(MarkerRole) and index.data(IncludeRole):
+        return _markerIcon(index.data(MarkerStyleRole)[MARKER_STYLE], index.data(MarkerStyleRole)[MARKER_COLOR])
       else:
         return self.pix
     elif role == ValueRole:
@@ -69,19 +88,39 @@ class WptModel(QtCore.QAbstractTableModel):
       return self.waypoints[index.row()]['ID']
     elif role == IncludeRole:
       return self.includeStates[index.row()]
-    elif role == SplitStateRole:
+    elif role == MarkerRole:
+      return self.markerStates[index.row()]
+    elif role == CaptionRole:
+      return self.captionStates[index.row()]
+    elif role == SplitLineRole:
       return self.splitStates[index.row()]
     elif role == NeglectRole:
       return self.neglectStates[index.row()]
-    elif role == MarkerRole:
-      return {k: self.pointStyles[index.row()][k] for k in {MARKER_COLOR, MARKER_STYLE, MARKER_SIZE}}
-    elif role == SplitLineRole:
-      return {k: self.pointStyles[index.row()][k] for k in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}}
-    elif role == CaptionRole:
-      return {k: self.pointStyles[index.row()][k] for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE}}
-    elif role == QtCore.Qt.BackgroundRole:
-      return INCCOLORS[self.includeStates[index.row()]]
-    elif role == QtCore.Qt.FontRole:
+    elif role == MarkerStyleRole:
+      if all([k in self.pointStyles[index.row()] for k in {MARKER_COLOR, MARKER_STYLE, MARKER_SIZE}]):
+        return {k: self.pointStyles[index.row()][k] for k in {MARKER_COLOR, MARKER_STYLE, MARKER_SIZE}}
+      else:
+        return {k: TheConfig.getValue('PointStyle', k) for k in {MARKER_COLOR, MARKER_STYLE, MARKER_SIZE}}
+    elif role == CaptionStyleRole:
+      if all([k in self.pointStyles[index.row()] for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE}]):
+        return {k: self.pointStyles[index.row()][k] for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE}}
+      else:
+        return {k: TheConfig.getValue('PointStyle', k) for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE}}
+    elif role == SplitLineStyleRole:
+      if all([k in self.pointStyles[index.row()] for k in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}]):
+        return {k: self.pointStyles[index.row()][k] for k in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}}
+      else:
+        return {k: TheConfig.getValue('PointStyle', k) for k in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}}
+    elif role == Qt.BackgroundRole:
+      if not self.includeStates[index.row()]:
+        return SkipColor
+      elif self.captionStates[index.row()]:
+        return CaptionColor
+      elif self.markerStates[index.row()]:
+        return MarkerColor
+      else:
+        return Qt.white
+    elif role == Qt.FontRole:
       font = QFont()
       if self.splitStates[index.row()]:
         font.setBold(True)
@@ -91,61 +130,70 @@ class WptModel(QtCore.QAbstractTableModel):
     return None
 
   def setData(self, index, value, role):
-    if index.isValid() and role == QtCore.Qt.EditRole and value != self.waypoints[index.row()][NAME]:
-      if index.row() in self.changedNames:
-        self.pointNames[self.changedNames.index(index.row())] = value
-      else:
-        self.changedNames += [index.row()]
-        self.pointNames += [value]
+    if index.isValid() and role == Qt.EditRole and value != self.waypoints[index.row()][NAME]:
+      self.changedNames[index.row()] = value
       self.dataChanged.emit(index, index)
-      self.namesChanged.emit(True)
+      self.wptDataChanged.emit()
       return True
     else:
       return False
 
   def flags(self, index):
     if index.column() == NAME:
-      return super(WptModel, self).flags(index) | QtCore.Qt.ItemIsEditable
+      return super(WptModel, self).flags(index) | Qt.ItemIsEditable
     else:
       return super(WptModel, self).flags(index)
 
   def headerData(self, section, orientation, role):
-    if role == QtCore.Qt.DisplayRole:
-      return self.fields[section] if orientation == QtCore.Qt.Horizontal else section + 1
+    if role == Qt.DisplayRole:
+      return self.fields[section] if orientation == Qt.Horizontal else section + 1
     return None
 
+  def parent(self, index=None):
+    if index is not None:
+      return QModelIndex()
+    else:
+      return super(WptModel, self).parent()
+
   def copyToClipboard(self, IDs):
     text = ''
     for i in IDs:
       text += '\t'.join([self.index(i, f).data() for f in WPTFIELDS]) + '\n'
     QGuiApplication.clipboard().setText(text)
 
-  def getIndexesWithIncludeState(self, state):
-    return [i for i, s in enumerate(self.includeStates) if s == state]
+  def getSkippedPoints(self):
+    return [i for i, s in enumerate(self.includeStates) if not s]
+
+  def getMarkedPoints(self):
+    return [i for i, s in enumerate(self.markerStates) if s]
+
+  def getCaptionedPoints(self):
+    return [i for i, s in enumerate(self.captionStates) if s]
+
+  def getSplitLines(self):
+    return [i for i, s in enumerate(self.splitStates) if s]
 
   def getNeglectStates(self):
     return [i for i, s in enumerate(self.neglectStates) if s]
 
   def getPointStyles(self, key):
     if key in {MARKER_COLOR, MARKER_STYLE, MARKER_SIZE}:
-      return [p[key] for i, p in enumerate(self.pointStyles) if self.includeStates[i] in {INC_MARKER, INC_CAPTION}]
+      return [p[key] for i, p in enumerate(self.pointStyles) if self.markerStates[i]]
+    if key in {CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE}:
+      return [p[key] for i, p in enumerate(self.pointStyles) if self.captionStates[i]]
     if key in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}:
       return [p[key] for i, p in enumerate(self.pointStyles) if self.splitStates[i]]
-    if key in {CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE}:
-      return [p[key] for i, p in enumerate(self.pointStyles) if self.includeStates[i] == INC_CAPTION]
-
-  def getSplitLines(self):
-    return [i for i, s in enumerate(self.splitStates) if s]
 
   def resetModel(self):
     self.beginResetModel()
     self.waypoints = []
     self.includeStates = []
+    self.markerStates = []
+    self.captionStates = []
     self.splitStates = []
     self.neglectStates = []
     self.pointStyles = []
-    self.changedNames = []
-    self.pointNames = []
+    self.changedNames = {}
     self.endResetModel()
 
   def setIncludeStates(self, IDs, state, update=True):
@@ -153,58 +201,86 @@ class WptModel(QtCore.QAbstractTableModel):
       self.includeStates[i] = state
     if update:
       self.parent().updatePoints()
+    self.wptDataChanged.emit()
+
+  def setMarkerStates(self, IDs, state):
+    for i in IDs:
+      self.markerStates[i] = state
+      for key in {MARKER_COLOR, MARKER_STYLE, MARKER_SIZE}:
+        if key not in self.pointStyles[i]:
+          self.pointStyles[i][key] = TheConfig.getValue('PointStyle', key)
+    self.wptDataChanged.emit()
+
+  def setCaptionStates(self, IDs, state):
+    for i in IDs:
+      self.captionStates[i] = state
+      for key in {CAPTION_POSX, CAPTION_POSY, CAPTION_SIZE}:
+        if key not in self.pointStyles[i]:
+          self.pointStyles[i][key] = TheConfig.getValue('PointStyle', key)
+    self.wptDataChanged.emit()
+
+  def setSplitLines(self, IDs, state):
+    for i in IDs:
+      self.splitStates[i] = state
+      for key in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}:
+        if key not in self.pointStyles[i]:
+          self.pointStyles[i][key] = TheConfig.getValue('PointStyle', key)
+    self.wptDataChanged.emit()
 
   def setNeglectStates(self, IDs, state):
     for i in IDs:
       self.neglectStates[i] = state
     self.parent().updateDistance()
+    self.wptDataChanged.emit()
 
   def setPointStyle(self, IDs, key, value):
     for i in IDs:
       self.pointStyles[i][key] = value
-
-  def setSplitLines(self, IDs, state):
-    for i in IDs:
-      self.splitStates[i] = state
+    self.wptDataChanged.emit()
 
   def resetNames(self, IDs):
     for i in IDs:
       if i in self.changedNames:
-        self.pointNames.pop(self.changedNames.index(i))
-        self.changedNames.remove(i)
+        del self.changedNames[i]
         self.dataChanged.emit(self.index(i, NAME), self.index(i, NAME))
-        self.namesChanged.emit(True)
+        self.wptDataChanged.emit()
 
-  namesChanged = QtCore.pyqtSignal(bool)
+  wptDataChanged = pyqtSignal()
 
 
-class TrkModel(QtCore.QAbstractTableModel):
+class TrkModel(QAbstractTableModel):
   def __init__(self, parent=None):
     super(TrkModel, self).__init__(parent)
     self.fields = [self.tr('Name'), self.tr('Segments'), self.tr('Points'), self.tr('Length'), self.tr('Time'), self.tr('Duration')]
     self.resetModel()
 
   def rowCount(self, parent=None):
-    return len(self.tracks)
+    if parent is not None and parent.isValid():
+      return 0
+    else:
+      return len(self.tracks)
 
   def columnCount(self, parent=None):
-    return len(self.fields)
+    if parent is not None and parent.isValid():
+      return 0
+    else:
+      return len(self.fields)
 
-  def data(self, index, role=QtCore.Qt.DisplayRole):
-    if role == QtCore.Qt.DisplayRole:
+  def data(self, index, role=Qt.DisplayRole):
+    if role == Qt.DisplayRole:
       if index.column() == TRKTIME and self.tracks[index.row()][index.column()] != '':
-        return str(self.tracks[index.row()][index.column()] + timedelta(minutes=TheConfig['ProfileStyle'].getint('TimeZoneOffset')))
+        return str(self.tracks[index.row()][index.column()] + timedelta(minutes=TheConfig.getValue('ProfileStyle', 'TimeZoneOffset')))
       else:
         return str(self.tracks[index.row()][index.column()])
     elif role == IncludeRole:
       return self.includeStates[index.row()]
-    elif role == QtCore.Qt.BackgroundRole:
-      return INCCOLORS[self.includeStates[index.row()]]
+    elif role == Qt.BackgroundRole:
+      return Qt.white if self.includeStates[index.row()] else SkipColor
     return None
 
   def headerData(self, section, orientation, role):
-    if role == QtCore.Qt.DisplayRole:
-      return self.fields[section] if orientation == QtCore.Qt.Horizontal else section + 1
+    if role == Qt.DisplayRole:
+      return self.fields[section] if orientation == Qt.Horizontal else section + 1
     return None
 
   def copyToClipboard(self, IDs):
@@ -213,8 +289,8 @@ class TrkModel(QtCore.QAbstractTableModel):
       text += '\t'.join([self.index(i, f).data() for f in TRKFIELDS]) + '\n'
     QGuiApplication.clipboard().setText(text)
 
-  def getIndexesWithIncludeState(self, state):
-    return [i for i, s in enumerate(self.includeStates) if s == state]
+  def getSkippedTracks(self):
+    return [i for i, s in enumerate(self.includeStates) if not s]
 
   def resetModel(self):
     self.beginResetModel()
@@ -229,7 +305,7 @@ class TrkModel(QtCore.QAbstractTableModel):
       self.parent().updatePoints()
 
 
-class GpxParser(QtCore.QObject):
+class GpxParser(QObject):
   def __init__(self, parent=None):
     super(GpxParser, self).__init__(parent)
     self.wptmodel = WptModel(self)
@@ -250,21 +326,17 @@ class GpxParser(QtCore.QObject):
       raise GpxWarning(filename + self.tr(' is an invalid GPX file.'))
     ns = {'ns': doc.getroot().tag.split('}')[0][1:]}
 
-    defaultStyle = {MARKER_COLOR: int(TheConfig['PointStyle']['MarkerColor']),
-                    MARKER_STYLE: TheConfig['PointStyle']['MarkerStyle'],
-                    MARKER_SIZE: int(TheConfig['PointStyle']['MarkerSize']),
-                    LINE_COLOR: int(TheConfig['PointStyle']['SplitLineColor']),
-                    LINE_STYLE: TheConfig['PointStyle']['SplitLineStyle'],
-                    LINE_WIDTH: float(TheConfig['PointStyle']['SplitLineWidth']),
-                    CAPTION_POSX: int(TheConfig['PointStyle']['CaptionPositionX']),
-                    CAPTION_POSY: int(TheConfig['PointStyle']['CaptionPositionY']),
-                    CAPTION_SIZE: int(TheConfig['PointStyle']['CaptionSize'])}
+    tag = 'name'
+    if TheConfig.getValue('ProfileStyle', 'ReadNameFromTag') == 1:  # Comment
+      tag = 'cmt'
+    elif TheConfig.getValue('ProfileStyle', 'ReadNameFromTag') == 2:  # Description
+      tag = 'desc'
 
-    wptid = len(self.wptmodel.waypoints)
+    wptid = self.wptmodel.rowCount()
     for p in doc.iterfind('.//{%(ns)s}wpt' % ns):
       try:
         point = {}
-        name = p.findtext('{%(ns)s}name' % ns)
+        name = p.findtext(('{%(ns)s}' + tag) % ns)
         point[NAME] = name.strip() if name is not None else ''
         point[LAT] = float(p.get('lat'))
         point[LON] = float(p.get('lon'))
@@ -286,12 +358,14 @@ class GpxParser(QtCore.QObject):
         point['DESC'] = p.findtext('{%(ns)s}desc' % ns)
         point['SYM'] = p.findtext('{%(ns)s}sym' % ns)
 
-        self.wptmodel.beginInsertRows(QtCore.QModelIndex(), wptid, wptid)
+        self.wptmodel.beginInsertRows(QModelIndex(), wptid, wptid)
         self.wptmodel.waypoints += [point]
-        self.wptmodel.includeStates += [INC_DEFAULT]
+        self.wptmodel.includeStates += [True]
+        self.wptmodel.markerStates += [False]
+        self.wptmodel.captionStates += [False]
         self.wptmodel.splitStates += [False]
         self.wptmodel.neglectStates += [False]
-        self.wptmodel.pointStyles += [defaultStyle.copy()]
+        self.wptmodel.pointStyles += [{}]
         self.wptmodel.endInsertRows()
         wptid += 1
 
@@ -299,7 +373,7 @@ class GpxParser(QtCore.QObject):
         self.warningSent.emit(self.tr('File read error'),
                               self.tr('Waypoint ') + (point[NAME] + ' ' if point[NAME] != '' else '') + self.tr('is invalid and will be skipped.', 'Waypoint'))
 
-    trkid = len(self.wptmodel.waypoints)
+    trkid = self.trkmodel.rowCount()
     for t in doc.iterfind('.//{%(ns)s}trk' % ns):
       try:
         track = {}
@@ -346,9 +420,9 @@ class GpxParser(QtCore.QObject):
         track[TRKDUR] = track['SEGMENTS'][-1][-1][TIME] - track['SEGMENTS'][0][0][TIME] \
                         if track['SEGMENTS'][0][0][TIME] != '' and track['SEGMENTS'][-1][-1][TIME] != '' else ''
 
-        self.trkmodel.beginInsertRows(QtCore.QModelIndex(), trkid, trkid)
+        self.trkmodel.beginInsertRows(QModelIndex(), trkid, trkid)
         self.trkmodel.tracks += [track]
-        self.trkmodel.includeStates += [INC_DEFAULT]
+        self.trkmodel.includeStates += [True]
         self.trkmodel.endInsertRows()
         trkid += 1
 
@@ -363,19 +437,19 @@ class GpxParser(QtCore.QObject):
     self.points = []
     includeTracks = True
     for i, p in enumerate(self.wptmodel.waypoints):
-      if self.wptmodel.includeStates[i] != INC_SKIP and p[TIME] == '':
+      if self.wptmodel.includeStates[i] and p[TIME] == '':
         includeTracks = False
 
-    if len(self.trkmodel.tracks) != 0 and includeTracks:
+    if self.trkmodel.rowCount() != 0 and includeTracks:
       for tind, track in enumerate(self.trkmodel.tracks):
-        if self.trkmodel.includeStates[tind] != INC_SKIP and \
-           (track[TRKTIME] != '' or len(self.wptmodel.waypoints) == len(self.wptmodel.getIndexesWithIncludeState(INC_SKIP))):
+        if self.trkmodel.includeStates[tind] and \
+           (track[TRKTIME] != '' or self.wptmodel.rowCount() == len(self.wptmodel.getSkippedPoints())):
           for sind, segment in enumerate(track['SEGMENTS']):
             self.points += zip([tind] * len(segment), [sind] * len(segment), range(len(segment)))
       ind = 0
       i = 0
-      while i < len(self.wptmodel.waypoints):
-        if self.wptmodel.includeStates[i] != INC_SKIP:
+      while i < self.wptmodel.rowCount():
+        if self.wptmodel.includeStates[i]:
           if ind < len(self.points) and self.wptmodel.waypoints[i][TIME] < self.trkmodel.tracks[self.points[ind][0]]['SEGMENTS'][self.points[ind][1]][self.points[ind][2]][TIME] or ind == len(self.points):
             self.points.insert(ind, i)
             i += 1
@@ -384,7 +458,7 @@ class GpxParser(QtCore.QObject):
           i += 1
     else:
       for i, p in enumerate(self.wptmodel.waypoints):
-        if self.wptmodel.includeStates[i] != INC_SKIP:
+        if self.wptmodel.includeStates[i]:
           self.points += [i]
 
     self.updateDistance()
@@ -394,7 +468,7 @@ class GpxParser(QtCore.QObject):
     prev_lat = None
     prev_lon = None
     dist = 0.0
-    dist_coeff = float(TheConfig['ProfileStyle']['DistanceCoefficient'])
+    dist_coeff = TheConfig.getValue('ProfileStyle', 'DistanceCoefficient')
 
     for p in self.points:
       if type(p) == int:
@@ -414,7 +488,7 @@ class GpxParser(QtCore.QObject):
         prev_lat = lat
         prev_lon = lon
 
-    for i in self.wptmodel.getIndexesWithIncludeState(INC_SKIP):
+    for i in self.wptmodel.getSkippedPoints():
       self.wptmodel.waypoints[i][DIST] = ''
 
     for i in self.trkmodel.tracks:
@@ -423,16 +497,16 @@ class GpxParser(QtCore.QObject):
   def updateTimeDifference(self):
     start_dt = None
     for i, p in enumerate(self.wptmodel.waypoints):
-      if self.wptmodel.includeStates[i] != INC_SKIP and p[TIME] != '':
+      if self.wptmodel.includeStates[i] and p[TIME] != '':
         start_dt = p[TIME]
         break
     for i, track in enumerate(self.trkmodel.tracks):
-      if self.trkmodel.includeStates[i] != INC_SKIP and track[TRKTIME] != '' and (start_dt is None or track[TRKTIME] < start_dt):
+      if self.trkmodel.includeStates[i] and track[TRKTIME] != '' and (start_dt is None or track[TRKTIME] < start_dt):
         start_dt = track[TRKTIME]
         break
 
     for i, p in enumerate(self.wptmodel.waypoints):
-      if self.wptmodel.includeStates[i] != INC_SKIP and p[TIME] != '':
+      if self.wptmodel.includeStates[i] and p[TIME] != '':
         p[TIMEDELTA] = p[TIME] - start_dt
         p[TIME_DAYS] = round(p[TIMEDELTA].days + p[TIMEDELTA].seconds / 60.0 / 60.0 / 24.0, 3)
       else:
@@ -440,7 +514,7 @@ class GpxParser(QtCore.QObject):
         p[TIME_DAYS] = ''
 
     for i, track in enumerate(self.trkmodel.tracks):
-      if self.trkmodel.includeStates[i] != INC_SKIP:
+      if self.trkmodel.includeStates[i]:
         for seg in track['SEGMENTS']:
           for p in seg:
             if p[TIME] != '':
@@ -461,7 +535,7 @@ class GpxParser(QtCore.QObject):
     maxlat = 0
     maxlon = -180
     for p, s in zip(self.wptmodel.waypoints, self.wptmodel.includeStates):
-      if s != INC_SKIP:
+      if s:
         element = ET.Element('wpt', attrib={'lat': str(p[LAT]), 'lon': str(p[LON])})
         root.append(element)
         if p[LAT] < minlat:
@@ -479,7 +553,10 @@ class GpxParser(QtCore.QObject):
         el.text = p[TIME].strftime('%Y-%m-%dT%H:%M:%SZ')
         element.append(el)
         el = ET.Element('name')
-        el.text = p[NAME]
+        if p['ID'] in self.wptmodel.changedNames:
+          el.text = self.wptmodel.changedNames[p['ID']]
+        else:
+          el.text = p[NAME]
         element.append(el)
         if p['CMT'] is not None:
           el = ET.Element('cmt')
@@ -495,7 +572,7 @@ class GpxParser(QtCore.QObject):
           element.append(el)
 
     for track, s in zip(self.trkmodel.tracks, self.trkmodel.includeStates):
-      if s != INC_SKIP:
+      if s:
         elTrk = ET.Element('trk')
         root.append(elTrk)
         el = ET.Element('name')
@@ -546,15 +623,21 @@ class GpxParser(QtCore.QObject):
     with open(filename, 'w', encoding='utf-8') as file:
       file.write(outgpx)
 
-  warningSent = QtCore.pyqtSignal(str, str)
+  warningSent = pyqtSignal(str, str)
 
 
-class GpxSortFilterModel(QtCore.QSortFilterProxyModel):
+class GpxSortFilterModel(QSortFilterProxyModel):
   def __init__(self, parent):
     super(GpxSortFilterModel, self).__init__(parent)
 
+    self.includeSkipped = True
+    self.includeMarked = True
+    self.includeCaptioned = True
+    self.includeMarkedCaptioned = True
+    self.includeOther = True
+
   def lessThan(self, left, right):
-    if (left.column() == right.column() == TIME) or (left.column() == right.column() == TIMEDELTA):
+    if left.column() == right.column() and right.column() != NAME:
       if left.data() != '' and right.data() != '':
         return left.data(ValueRole) < right.data(ValueRole)
       elif right.data() == '':
@@ -564,6 +647,25 @@ class GpxSortFilterModel(QtCore.QSortFilterProxyModel):
 
     return super(GpxSortFilterModel, self).lessThan(left, right)
 
+  def filterAcceptsRow(self, source_row, source_parent):
+    ind = self.sourceModel().index(source_row, NAME, source_parent)
+    if (ind.data(IncludeRole) or self.includeSkipped) and \
+       (not ind.data(MarkerRole) or ind.data(CaptionRole) or self.includeMarked) and \
+       (not ind.data(CaptionRole) or ind.data(MarkerRole) or self.includeCaptioned) and \
+       (not ind.data(MarkerRole) or not ind.data(CaptionRole) or self.includeMarkedCaptioned) and \
+       (not ind.data(IncludeRole) or ind.data(MarkerRole) or ind.data(CaptionRole) or self.includeOther):
+      return super(GpxSortFilterModel, self).filterAcceptsRow(source_row, source_parent)
+    else:
+      return False
+
+  def setFilterMask(self, skipped, marked, captioned, markedCaptioned, other):
+    self.includeSkipped = skipped
+    self.includeMarked = marked
+    self.includeCaptioned = captioned
+    self.includeMarkedCaptioned = markedCaptioned
+    self.includeOther = other
+    self.invalidateFilter()
+
 
 def _distance(lat1, lon1, lat2, lon2):
   Radius = 6378.14  # The equatorial radius of the Earth
@@ -577,71 +679,132 @@ def _distance(lat1, lon1, lat2, lon2):
   return dist
 
 
-def _markerIcon(style, color):
-  size = 16
-  r = 2 * size / 5
-  pix = QPixmap(size, size)
-  pix.fill(QtCore.Qt.transparent)
-  p = QPainter(pix)
-  p.setRenderHint(QPainter.Antialiasing)
-  p.setPen(QColor(color))
-  p.setBrush(QColor(color))
-  center = QtCore.QPoint(size/2, size/2)
+def markerPath(style, size):
+  r = size / 2.0
+  center = QPointF(r, r)
+  path = QPainterPath()
 
   if style == '.':
-    p.drawEllipse(QtCore.QPoint(size/2, size/2), 3, 3)
+    path.addEllipse(center, 3, 3)
   elif style == ',':
-    #p.drawPoint(QtCore.QPoint(size/2, size/2))
-    p.drawEllipse(QtCore.QPoint(size/2, size/2), 1, 1)
+    path.addEllipse(center, 1, 1)
   elif style == 'o':
-    p.drawEllipse(QtCore.QPoint(size/2, size/2), size/3, size/3)
+    path.addEllipse(center, 0.8 * r,  0.8 * r)
   elif style == 'v':
-    p.drawPolygon(center + r * QtCore.QPointF(cos(pi/2), sin(pi/2)), center + r * QtCore.QPointF(cos(7*pi/6), sin(7*pi/6)), center + r * QtCore.QPointF(cos(11*pi/6), sin(11*pi/6)))
+    path.addPolygon(QPolygonF([center + r * QPointF(cos(pi/2), sin(pi/2)),
+                               center + r * QPointF(cos(7*pi/6), sin(7*pi/6)),
+                               center + r * QPointF(cos(11*pi/6), sin(11*pi/6))]))
   elif style == '^':
-    p.drawPolygon(center + r * QtCore.QPointF(cos(3*pi/2), sin(3*pi/2)), center + r * QtCore.QPointF(cos(pi/6), sin(pi/6)), center + r * QtCore.QPointF(cos(5*pi/6), sin(5*pi/6)))
+    path.addPolygon(QPolygonF([center + r * QPointF(cos(3*pi/2), sin(3*pi/2)),
+                               center + r * QPointF(cos(pi/6), sin(pi/6)),
+                               center + r * QPointF(cos(5*pi/6), sin(5*pi/6))]))
   elif style == '<':
-    p.drawPolygon(center + r * QtCore.QPointF(cos(pi), sin(pi)), center + r * QtCore.QPointF(cos(5*pi/3), sin(5*pi/3)), center + r * QtCore.QPointF(cos(pi/3), sin(pi/3)))
+    path.addPolygon(QPolygonF([center + r * QPointF(cos(pi), sin(pi)),
+                               center + r * QPointF(cos(5*pi/3), sin(5*pi/3)),
+                               center + r * QPointF(cos(pi/3), sin(pi/3))]))
   elif style == '>':
-    p.drawPolygon(center + r * QtCore.QPointF(cos(0), sin(0)), center + r * QtCore.QPointF(cos(2*pi/3), sin(2*pi/3)), center + r * QtCore.QPointF(cos(4*pi/3), sin(4*pi/3)))
+    path.addPolygon(QPolygonF([center + r * QPointF(cos(0), sin(0)),
+                               center + r * QPointF(cos(2*pi/3), sin(2*pi/3)),
+                               center + r * QPointF(cos(4*pi/3), sin(4*pi/3))]))
   elif style == '1':
-    p.drawLine(center, center + r * QtCore.QPointF(cos(pi/2), sin(pi/2)))
-    p.drawLine(center, center + r * QtCore.QPointF(cos(7*pi/6), sin(7*pi/6)))
-    p.drawLine(center, center + r * QtCore.QPointF(cos(11*pi/6), sin(11*pi/6)))
+    path.moveTo(center)
+    path.lineTo(center + r * QPointF(cos(pi/2), sin(pi/2)))
+    path.moveTo(center)
+    path.lineTo(center + r * QPointF(cos(7*pi/6), sin(7*pi/6)))
+    path.moveTo(center)
+    path.lineTo(center + r * QPointF(cos(11*pi/6), sin(11*pi/6)))
   elif style == '2':
-    p.drawLine(center, center - r * QtCore.QPointF(cos(pi/2), sin(pi/2)))
-    p.drawLine(center, center - r * QtCore.QPointF(cos(7*pi/6), sin(7*pi/6)))
-    p.drawLine(center, center - r * QtCore.QPointF(cos(11*pi/6), sin(11*pi/6)))
+    path.moveTo(center)
+    path.lineTo(center - r * QPointF(cos(pi/2), sin(pi/2)))
+    path.moveTo(center)
+    path.lineTo(center - r * QPointF(cos(7*pi/6), sin(7*pi/6)))
+    path.moveTo(center)
+    path.lineTo(center - r * QPointF(cos(11*pi/6), sin(11*pi/6)))
   elif style == '3':
-    p.drawLine(center, center + r * QtCore.QPointF(cos(pi), sin(pi)))
-    p.drawLine(center, center + r * QtCore.QPointF(cos(pi/3), sin(pi/3)))
-    p.drawLine(center, center + r * QtCore.QPointF(cos(5*pi/3), sin(5*pi/3)))
+    path.moveTo(center)
+    path.lineTo(center + r * QPointF(cos(pi), sin(pi)))
+    path.moveTo(center)
+    path.lineTo(center + r * QPointF(cos(pi/3), sin(pi/3)))
+    path.moveTo(center)
+    path.lineTo(center + r * QPointF(cos(5*pi/3), sin(5*pi/3)))
   elif style == '4':
-    p.drawLine(center, center + r * QtCore.QPointF(cos(0), sin(0)))
-    p.drawLine(center, center + r * QtCore.QPointF(cos(2*pi/3), sin(2*pi/3)))
-    p.drawLine(center, center + r * QtCore.QPointF(cos(4*pi/3), sin(4*pi/3)))
+    path.moveTo(center)
+    path.lineTo(center + r * QPointF(cos(0), sin(0)))
+    path.moveTo(center)
+    path.lineTo(center + r * QPointF(cos(2*pi/3), sin(2*pi/3)))
+    path.moveTo(center)
+    path.lineTo(center + r * QPointF(cos(4*pi/3), sin(4*pi/3)))
   elif style == 's':
-    p.drawPolygon(center + QtCore.QPointF(r, r), center + QtCore.QPointF(-r, r), center - QtCore.QPointF(r, r), center + QtCore.QPointF(r, -r))
+    path.addRect((1 - sqrt(0.5)) * r, (1 - sqrt(0.5)) * r, sqrt(2.0) * r, sqrt(2.0) * r)
   elif style == 'p':
-    p.drawPolygon(center + r * QtCore.QPointF(cos(3*pi/2), sin(3*pi/2)), center + r * QtCore.QPointF(cos(19*pi/10), sin(19*pi/10)), center + r * QtCore.QPointF(cos(3*pi/10), sin(3*pi/10)), center + r * QtCore.QPointF(cos(7*pi/10), sin(7*pi/10)), center + r * QtCore.QPointF(cos(11*pi/10), sin(11*pi/10)))
+    path.addPolygon(QPolygonF([center + r * QPointF(cos(3*pi/2), sin(3*pi/2)),
+                               center + r * QPointF(cos(19*pi/10), sin(19*pi/10)),
+                               center + r * QPointF(cos(3*pi/10), sin(3*pi/10)),
+                               center + r * QPointF(cos(7*pi/10), sin(7*pi/10)),
+                               center + r * QPointF(cos(11*pi/10), sin(11*pi/10))]))
   elif style == '*':
-    p.drawPolygon(center + r * QtCore.QPointF(cos(3*pi/2), sin(3*pi/2)), center + r/2 * QtCore.QPointF(cos(17*pi/10), sin(17*pi/10)), center + r * QtCore.QPointF(cos(19*pi/10), sin(19*pi/10)), center + r/2 * QtCore.QPointF(cos(pi/10), sin(pi/10)), center + r * QtCore.QPointF(cos(3*pi/10), sin(3*pi/10)), center + r/2 * QtCore.QPointF(cos(pi/2), sin(pi/2)), center + r * QtCore.QPointF(cos(7*pi/10), sin(7*pi/10)), center + r/2 * QtCore.QPointF(cos(9*pi/10), sin(9*pi/10)), center + r * QtCore.QPointF(cos(11*pi/10), sin(11*pi/10)), center + r/2 * QtCore.QPointF(cos(13*pi/10), sin(13*pi/10)))
+    path.addPolygon(QPolygonF([center + r * QPointF(cos(3*pi/2), sin(3*pi/2)),
+                               center + r/2 * QPointF(cos(17*pi/10), sin(17*pi/10)),
+                               center + r * QPointF(cos(19*pi/10), sin(19*pi/10)),
+                               center + r/2 * QPointF(cos(pi/10), sin(pi/10)),
+                               center + r * QPointF(cos(3*pi/10), sin(3*pi/10)),
+                               center + r/2 * QPointF(cos(pi/2), sin(pi/2)),
+                               center + r * QPointF(cos(7*pi/10), sin(7*pi/10)),
+                               center + r/2 * QPointF(cos(9*pi/10), sin(9*pi/10)),
+                               center + r * QPointF(cos(11*pi/10), sin(11*pi/10)),
+                               center + r/2 * QPointF(cos(13*pi/10), sin(13*pi/10))]))
   elif style == 'h':
-    p.drawPolygon(center + r * QtCore.QPointF(cos(pi/6), sin(pi/6)), center + r * QtCore.QPointF(cos(pi/2), sin(pi/2)), center + r * QtCore.QPointF(cos(5*pi/6), sin(5*pi/6)), center + r * QtCore.QPointF(cos(7*pi/6), sin(7*pi/6)), center + r * QtCore.QPointF(cos(3*pi/2), sin(3*pi/2)), center + r * QtCore.QPointF(cos(11*pi/6), sin(11*pi/6)))
+    path.addPolygon(QPolygonF([center + r * QPointF(cos(pi/6), sin(pi/6)),
+                               center + r * QPointF(cos(pi/2), sin(pi/2)),
+                               center + r * QPointF(cos(5*pi/6), sin(5*pi/6)),
+                               center + r * QPointF(cos(7*pi/6), sin(7*pi/6)),
+                               center + r * QPointF(cos(3*pi/2), sin(3*pi/2)),
+                               center + r * QPointF(cos(11*pi/6), sin(11*pi/6))]))
   elif style == 'H':
-    p.drawPolygon(center + r * QtCore.QPointF(cos(0), sin(0)), center + r * QtCore.QPointF(cos(pi/3), sin(pi/3)), center + r * QtCore.QPointF(cos(2*pi/3), sin(2*pi/3)), center + r * QtCore.QPointF(cos(pi), sin(pi)), center + r * QtCore.QPointF(cos(4*pi/3), sin(4*pi/3)), center + r * QtCore.QPointF(cos(5*pi/3), sin(5*pi/3)))
+    path.addPolygon(QPolygonF([center + r * QPointF(cos(0), sin(0)),
+                               center + r * QPointF(cos(pi/3), sin(pi/3)),
+                               center + r * QPointF(cos(2*pi/3), sin(2*pi/3)),
+                               center + r * QPointF(cos(pi), sin(pi)),
+                               center + r * QPointF(cos(4*pi/3), sin(4*pi/3)),
+                               center + r * QPointF(cos(5*pi/3), sin(5*pi/3))]))
   elif style == '+':
-    p.drawLine(center + QtCore.QPointF(0, r), center - QtCore.QPointF(0, r))
-    p.drawLine(center + QtCore.QPointF(r, 0), center - QtCore.QPointF(r, 0))
+    path.moveTo(r, 0)
+    path.lineTo(r, size)
+    path.moveTo(0, r)
+    path.lineTo(size, r)
   elif style == 'x':
-    p.drawLine(center + QtCore.QPointF(r, r), center - QtCore.QPointF(r, r))
-    p.drawLine(center + QtCore.QPointF(r, -r), center + QtCore.QPointF(-r, r))
+    path.moveTo((1 - sqrt(0.5)) * r, (1 - sqrt(0.5)) * r)
+    path.lineTo((1 + sqrt(0.5)) * r, (1 + sqrt(0.5)) * r)
+    path.moveTo((1 + sqrt(0.5)) * r, (1 - sqrt(0.5)) * r)
+    path.lineTo((1 - sqrt(0.5)) * r, (1 + sqrt(0.5)) * r)
   elif style == 'D':
-    p.drawPolygon(center + QtCore.QPointF(r, 0), center + QtCore.QPointF(0, r), center - QtCore.QPointF(r, 0), center - QtCore.QPointF(0, r))
+    path.addPolygon(QPolygonF([center + QPointF(r, 0), center + QPointF(0, r),
+                               center - QPointF(r, 0), center - QPointF(0, r)]))
   elif style == 'd':
-    p.drawPolygon(center + QtCore.QPointF(r/2, 0), center + QtCore.QPointF(0, r), center - QtCore.QPointF(r/2, 0), center - QtCore.QPointF(0, r))
+    path.addPolygon(QPolygonF([center + QPointF(0.6 * r, 0), center + QPointF(0, r),
+                               center - QPointF(0.6 * r, 0), center - QPointF(0, r)]))
   elif style == '|':
-    p.drawLine(center + QtCore.QPointF(0, r), center - QtCore.QPointF(0, r))
+    path.moveTo(r, 0)
+    path.lineTo(r, size)
   elif style == '_':
-    p.drawLine(center + QtCore.QPointF(r, 0), center - QtCore.QPointF(r, 0))
+    path.moveTo(0, r)
+    path.lineTo(size, r)
+
+  path.closeSubpath()
+  return path.translated(-center)
+
+
+def _markerIcon(style, color):
+  size = 16
+  pix = QPixmap(size, size)
+  pix.fill(Qt.transparent)
+  p = QPainter(pix)
+  p.setRenderHint(QPainter.Antialiasing)
+  if style in {'1', '2', '3', '4', '+', 'x', '_', '|'}:
+    p.setPen(QPen(QColor(color), 3, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
+  else:
+    p.setPen(QPen(QColor(color), 1, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
+  p.setBrush(QColor(color))
+  p.drawPath(markerPath(style, 0.8 * size).translated(QPointF(size / 2.0, size / 2.0)))
 
   return pix
