@@ -1,6 +1,6 @@
 # gpxviewer
 #
-# Copyright (C) 2017 Sergey Salnikov <salsergey@gmail.com>
+# Copyright (C) 2017-2019 Sergey Salnikov <salsergey@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3
@@ -14,10 +14,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from datetime import datetime, timedelta
-from PyQt5 import QtWidgets
-from PyQt5.QtCore import Qt, QRegularExpression
+from datetime import timedelta
+from PyQt5.QtCore import Qt, QRegularExpression, pyqtSlot
 from PyQt5.QtGui import QGuiApplication, QIcon
+from PyQt5.QtWidgets import QHBoxLayout, QLabel, QLineEdit, QMainWindow, QSizePolicy, QSpacerItem, QTableWidgetItem, QWidget
 import gpxviewer.gpxmodel as gpx
 from gpxviewer.configstore import TheConfig
 from gpxviewer.gpxdocument import TheDocument
@@ -27,7 +27,7 @@ import gpxviewer.ui_statwindow
 NAME, DIST, RAISE, DROP, TIME = range(5)
 
 
-class StatWindow(QtWidgets.QMainWindow):
+class StatWindow(QMainWindow):
   def __init__(self, parent=None):
     super(StatWindow, self).__init__(parent)
     self.ui = gpxviewer.ui_statwindow.Ui_StatWindow()
@@ -35,12 +35,12 @@ class StatWindow(QtWidgets.QMainWindow):
 
     self.setWindowIcon(QIcon(':/icons/gpxviewer.svg'))
 
-    wdg = QtWidgets.QWidget()
-    wdg.setLayout(QtWidgets.QHBoxLayout())
-    wdg.layout().addItem(QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding))
-    self.filterLabel = QtWidgets.QLabel(self.tr('By name:'))
+    wdg = QWidget()
+    wdg.setLayout(QHBoxLayout())
+    wdg.layout().addItem(QSpacerItem(40, 20, QSizePolicy.Expanding))
+    self.filterLabel = QLabel(self.tr('By name:'))
     wdg.layout().addWidget(self.filterLabel)
-    self.filterLineEdit = QtWidgets.QLineEdit()
+    self.filterLineEdit = QLineEdit()
     self.filterLineEdit.setMinimumWidth(200)
     self.filterLineEdit.setPlaceholderText(self.tr('Enter regular expression'))
     self.filterLineEdit.setClearButtonEnabled(True)
@@ -80,6 +80,7 @@ class StatWindow(QtWidgets.QMainWindow):
     TheConfig['StatWindow']['BySplittingLines'] = str(checked)
     self.updateStatistics()
 
+  @pyqtSlot()
   def updateStatistics(self):
     self.ui.statWidget.clearContents()
     n = 0
@@ -87,27 +88,27 @@ class StatWindow(QtWidgets.QMainWindow):
     alt_drop = 0
 
     for i in range(TheDocument.wptmodel.rowCount()):
-      if TheDocument.wptmodel.index(i, 0).data(gpx.IncludeRole) != gpx.INC_SKIP:
+      if TheDocument.wptmodel.index(i, 0).data(gpx.IncludeRole):
         point_start = TheDocument.wptmodel.waypoints[i]
         point_prev = point_start
         break
     for i in range(TheDocument.wptmodel.rowCount() - 1, -1, -1):
-      if TheDocument.wptmodel.index(i, 0).data(gpx.IncludeRole) != gpx.INC_SKIP:
+      if TheDocument.wptmodel.index(i, 0).data(gpx.IncludeRole):
         i_stop = i
         break
 
     segments = []
     for i, p in enumerate(TheDocument.wptmodel.waypoints):
-      if TheDocument.wptmodel.index(i, 0).data(gpx.IncludeRole) != gpx.INC_SKIP:
-        if self.ui.actionBySplittingLines.isChecked() and TheDocument.wptmodel.index(i, 0).data(gpx.SplitStateRole) or \
+      if TheDocument.wptmodel.index(i, 0).data(gpx.IncludeRole):
+        if self.ui.actionBySplittingLines.isChecked() and TheDocument.wptmodel.index(i, 0).data(gpx.SplitLineRole) or \
            not self.ui.actionBySplittingLines.isChecked() and QRegularExpression(self.filterLineEdit.text()).match(p[gpx.NAME]).hasMatch() or \
            i == i_stop:
-             if p['ID'] != point_start['ID']:
-              segments += [i]
+          if p['ID'] != point_start['ID']:
+            segments += [i]
     self.ui.statWidget.setRowCount(len(segments))
 
     for i, p in enumerate(TheDocument.wptmodel.waypoints):
-      if TheDocument.wptmodel.index(i, 0).data(gpx.IncludeRole) != gpx.INC_SKIP:
+      if TheDocument.wptmodel.index(i, 0).data(gpx.IncludeRole):
         if p[gpx.ALT] > point_prev[gpx.ALT]:
           alt_raise += p[gpx.ALT] - point_prev[gpx.ALT]
         else:
@@ -115,11 +116,11 @@ class StatWindow(QtWidgets.QMainWindow):
         point_prev = p
 
         if i in segments:
-          self.ui.statWidget.setItem(n, NAME, QtWidgets.QTableWidgetItem(point_start[gpx.NAME] + ' — ' + p[gpx.NAME]))
-          self.ui.statWidget.setItem(n, DIST, QtWidgets.QTableWidgetItem(str(round(p[gpx.DIST] - point_start[gpx.DIST], 3))))
-          self.ui.statWidget.setItem(n, RAISE, QtWidgets.QTableWidgetItem(str(alt_raise)))
-          self.ui.statWidget.setItem(n, DROP, QtWidgets.QTableWidgetItem(str(alt_drop)))
-          time_item = QtWidgets.QTableWidgetItem(str(p[gpx.TIME] - point_start[gpx.TIME]))
+          self.ui.statWidget.setItem(n, NAME, QTableWidgetItem(point_start[gpx.NAME] + ' — ' + p[gpx.NAME]))
+          self.ui.statWidget.setItem(n, DIST, QTableWidgetItem(str(round(p[gpx.DIST] - point_start[gpx.DIST], 3))))
+          self.ui.statWidget.setItem(n, RAISE, QTableWidgetItem(str(alt_raise)))
+          self.ui.statWidget.setItem(n, DROP, QTableWidgetItem(str(alt_drop)))
+          time_item = QTableWidgetItem(str(p[gpx.TIME] - point_start[gpx.TIME]))
           time_item.setData(Qt.UserRole, p[gpx.TIME] - point_start[gpx.TIME])
           self.ui.statWidget.setItem(n, TIME, time_item)
           point_start = p
@@ -129,6 +130,7 @@ class StatWindow(QtWidgets.QMainWindow):
 
     self.ui.statWidget.resizeColumnsToContents()
 
+  @pyqtSlot()
   def updateTotalStatistics(self):
     total_dist = 0
     total_raise = 0
@@ -137,8 +139,11 @@ class StatWindow(QtWidgets.QMainWindow):
 
     if self.ui.statWidget.selectionModel().hasSelection():
       rows = [i.row() for i in self.ui.statWidget.selectionModel().selectedRows()]
+      self.ui.totalGroupBox.setTitle(self.tr('Total (for selected segments)'))
     else:
       rows = range(self.ui.statWidget.rowCount())
+      self.ui.totalGroupBox.setTitle(self.tr('Total'))
+
     for i in rows:
       total_dist += round(float(self.ui.statWidget.item(i, DIST).data(Qt.DisplayRole)), 3)
       total_raise += int(self.ui.statWidget.item(i, RAISE).data(Qt.DisplayRole))
